/*	$NetBSD$	*/

/*
 * Copyright (c) 2001 ITOH Yasufumi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/endian.h>
#include <string.h>
#include <errno.h>

#include "dcmem.h"

const char *
dcmem_strerror(eno)
	int eno;
{

	switch (eno) {
	case DCMEM_ESYS:
		return strerror(errno);

	case 0:
		return "no error";

	case DCMEM_EFORMAT:
		return "not formatted";
	case DCMEM_ENOTSUPP:
		return "un-supported format";
	case DCMEM_ERBLK:
		return "root block corrupt";
	case DCMEM_EINCONSISTENT:
		return "structure inconsistent";
	case DCMEM_ERDONLY:
		return "write requeted on read-only mode";
	case DCMEM_EUXEOF:
		return "unexpected end-of-file";
	case DCMEM_EHWPROBLEM:
		return "memory card hardware problem";
	case DCMEM_ENOENT:
		return "file not found";
	case DCMEM_EEXISTS:
		return "file already exists";
	case DCMEM_ENOSPC:
		return "not enough space";
	case DCMEM_EDIRFULL:
		return "no free directory slot";
	case DCMEM_EINVAL:
		return "invalid argument";
	default:
		break;
	}
	return "unknown error";
}
