/*	$NetBSD$	*/

/*
 * Copyright (c) 2001 ITOH Yasufumi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/endian.h>

#include "dcmem.h"

int
dcmem_fat_find_free_data(m)
	const struct dcmem *m;
{
	int i;

	for (i = m->dm_minblk + m->dm_nblk - 1; i >= 0; i--) {
		if (DCMEM_FAT_IS_FREE(m, (unsigned) i))
			return i;
	}

	return -1;
}

int
dcmem_fat_cnt_free_data(m)
	const struct dcmem *m;
{
	int i;
	int cnt;

	cnt = 0;
	for (i = m->dm_minblk + m->dm_nblk - 1; i >= 0; i--) {
		if (DCMEM_FAT_IS_FREE(m, (unsigned) i))
			cnt++;
	}

	return cnt;
}

int
dcmem_fat_cnt_free_program(m)
	const struct dcmem *m;
{
	int i;

	for (i = DCMEM_START_PROGRAM;
	    i < DCMEM_SIZE_PROGRAM && DCMEM_FAT_IS_FREE(m, (unsigned)i); i++)
		;

	return i;
}

int
dcmem_fat_find_free_sys(m)
	const struct dcmem *m;
{
	int i;

	for (i = m->dm_maxblk - 1; i >= m->dm_minblk + m->dm_nblk; i--) {
		if (DCMEM_FAT_IS_FREE(m, (unsigned) i))
			return i;
	}

	return -1;
}

int
dcmem_fat_cnt_free_sys(m)
	const struct dcmem *m;
{
	int i;
	int cnt;

	cnt = 0;
	for (i = m->dm_maxblk - 1; i >= m->dm_minblk + m->dm_nblk; i--) {
		if (DCMEM_FAT_IS_FREE(m, (unsigned) i))
			cnt++;
	}

	return cnt;
}

int
dcmem_fat_next(m, fatno)
	const struct dcmem *m;
	int fatno;
{
	int i;

	i = le16toh(m->dm_fat[fatno]);
	if (i == DCMEM_FAT_END)
		return -1;

	return i;
}

void
dcmem_fat_free_chain_data(m, fatno)
	const struct dcmem *m;
	int fatno;
{
	int next;

	while (fatno >= 0 && fatno < m->dm_minblk + m->dm_nblk) {
		next = dcmem_fat_next(m, fatno);
		DCMEM_FAT_MARK(m, fatno, DCMEM_FAT_FREE);
		fatno = next;
	}
}
