/*	$NetBSD$	*/

/*
 * Copyright (c) 2001 ITOH Yasufumi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/endian.h>

#include "dcmem.h"

int
dcmem_write_file_dirent(m, d0, buf)
	struct dcmem *m;
	const struct dcdirent *d0;
	void *buf;
{
	int top, sz;
	int s;
	struct dcdirent *d;

	/*
	 * make sure we have an directory slot
	 */
	if ((d = dcmem_dir_alloc(m, d0->dd_name, &s)) == 0)
		return s;

	/*
	 * write the contents
	 */
	sz = le16toh(d0->dd_size);
	switch (d0->dd_type) {
	case DCMEM_FTYPE_DATA:
		s = dcmem_write_data(m, buf, sz, &top);
		break;
	case DCMEM_FTYPE_PROGRAM:
		s = dcmem_write_program(m, buf, sz);
		top = DCMEM_START_PROGRAM;
		break;
	default:
		return DCMEM_EINVAL;
	}

	if (s)
		return s;

	*d = *d0;
	d->dd_start = htole16(top);

	return 0;
}
